﻿--[[---------------------------------------------------------------------------

Copyright (c) 2008 by K. Scott Piel 
All Rights Reserved

E-mail: < kscottpiel@gmail.com >
Web:    < http://www.scottpiel.com >

This file is part of nUI.

    nUI is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    nUI is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with nUI.  If not, see <http://www.gnu.org/licenses/>.
	
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

--]]---------------------------------------------------------------------------

if not nUI then nUI = {}; end
if not nUI_DefaultConfig then nUI_DefaultConfig = {}; end
if not nUI_Profile then nUI_Profile = {}; end;

local CreateFrame         = CreateFrame;
local GetBindingKey       = GetBindingKey;
local RegisterStateDriver = RegisterStateDriver;

nUI_Profile.nUI_ButtonBar = {};

local ProfileCounter = nUI_Profile.nUI_ButtonBar;

-------------------------------------------------------------------------------
-- default button bar configurations

nUI_DefaultConfig.ButtonBars =
{
	["nUI_ActionBar"] =
	{
		btn_size = 54,
		gap      = 2,
		anchor   = "BOTTOM",
		xOfs     = 0,
		yOfs     = 123,
		rows     = 1,
		cols     = 12,
		page     = 1,
		nuibind  = "nUI_ACTIONBAR",
		binding  = "ACTIONBUTTON",
		label    = nUI_L["nUI_ActionBar"],
	},

	-- the Bliz "bottom left" bar

	["nUI_TopLeftBar"] =
	{		
		btn_size = 54,
		gap      = 2,
		anchor   = "TOPLEFT",
		yOfs     = 117,
		xOfs     = -680,
		rows     = 1,
		cols     = 12,
		page     = 6,
		nuibind  = "nUI_TOPLEFTBAR",
		binding  = "MULTIACTIONBAR1BUTTON",
		label    = nUI_L["nUI_TopLeftBar"],
	},
	
	-- the Bliz "bottom right" bar

	["nUI_TopRightBar"] =
	{		
		btn_size = 54,
		gap      = 2,
		anchor   = "TOPRIGHT",
		yOfs     = 117,
		xOfs     = 680,
		rows     = 1,
		cols     = 12,
		page     = 5,
		nuibind  = "nUI_TOPRIGHTBAR",
		binding  = "MULTIACTIONBAR2BUTTON",
		label    = nUI_L["nUI_TopRightBar"],
	},
	
	-- the Bliz "right 1" bar (aka "left" bar in the Bliz code)

	["nUI_LeftUnitBar"] =
	{		
		btn_size = 48,
		gap      = 2,
		anchor   = "BOTTOMLEFT",		
		yOfs     = -250,
		xOfs     = -680,
		rows     = 6,
		cols     = 2,
		page     = 3,
		nuibind  = "nUI_LEFTUNITBAR",
		binding  = "MULTIACTIONBAR4BUTTON",
		label    = nUI_L["nUI_LeftUnitBar"],
	},
		
	-- the Bliz "right 2" bar (aka "right" bar in the Bliz code)
	
	["nUI_RightUnitBar"] =
	{		
		btn_size = 48,
		gap      = 2,
		anchor   = "BOTTOMRIGHT",
		yOfs     = -250,
		xOfs     = 680,
		rows     = 6,
		cols     = 2,
		page     = 4,
		nuibind  = "nUI_RIGHTUNITBAR",
		binding  = "MULTIACTIONBAR3BUTTON",
		label    = nUI_L["nUI_RightUnitBar"],
	},

	-- bonus bar 1

	["nUI_BottomLeftBar"] =
	{		
		btn_size = 38,
		gap      = 2,
		anchor   = "BOTTOMLEFT",
		yOfs     = -223,
		xOfs     = -1270,
		rows     = 1,
		cols     = 12,
		page     = 2,
		nuibind  = "nUI_BOTTOMLEFTBAR",
		binding  = nil,
		label    = nUI_L["nUI_BottomLeftBar"],
	},
		
	-- bonus bar 2
	
	["nUI_BottomRightBar"] =
	{		
		btn_size = 38,
		gap      = 2,
		anchor   = "BOTTOMRIGHT",
		yOfs     = -223,
		xOfs     = 1270,
		rows     = 1,
		cols     = 12,
		base_id  = 108,
		nuibind  = "nUI_BOTTOMRIGHTBAR",
		binding  = nil,
		label    = nUI_L["nUI_BottomRightBar"],
	},	
};
	
-------------------------------------------------------------------------------
-- button bar event management

nUI_ButtonBars       = CreateFrame( "Frame", "nUI_ButtonBars", WorldFrame );
nUI_ButtonBars.Bars  = {};

function onButtonBarEvent()
		
	nUI_ProfileStart( ProfileCounter, "onButtonBarEvent", event );
	
	-- reset the button bar key bindings to Bliz's action bars on log out so users do
	-- not lose their key bindings if they disable or stop using nUI
	
	if event == "PLAYER_LOGOUT" then
	
		for i in pairs( nUI_ButtonBars.Bars ) do
			
			local bar = nUI_ButtonBars.Bars[i];

			for row in pairs( bar.Buttons ) do
				
				for col in pairs( bar.Buttons[row] ) do
					
					local button      = bar.Buttons[row][col];
							
					if button.bindingType then

						local btnName     = button:GetName();
						local actionName  = button.bindingType..(button.actionID or "");
						local key1, key2  = GetBindingKey( "CLICK "..btnName..":LeftButton" );
						
						if key1 then 
							SetBinding( key1, actionName );
						end
						if key2 then 
							SetBinding( key2, actionName );
						end
					end
				end
			end			
		end

		if KeyBindingFrame then
			
			if ( KeyBindingFrameCharacterButton:GetChecked() ) then
				KeyBindingFrame.which = CHARACTER_BINDINGS;
			else
				KeyBindingFrame.which = ACCOUNT_BINDINGS;
				if ( GetCurrentBindingSet() == CHARACTER_BINDINGS ) then
					if ( not CONFIRMED_DELETING_CHARACTER_SPECIFIC_BINDINGS ) then
						StaticPopup_Show("CONFIRM_DELETING_CHARACTER_SPECIFIC_BINDINGS");
						return;
					end
				end
			end
			
			SaveBindings(KeyBindingFrame.which);
		end
		
	-- set up the key bindings
	
	elseif event == "UPDATE_BINDINGS" then

		nUI_ButtonBars:UnregisterEvent( "UPDATE_BINDINGS" );
		
		for i in pairs( nUI_ButtonBars.Bars ) do
			
			local bar = nUI_ButtonBars.Bars[i];

			for row in pairs( bar.Buttons ) do
				
				for col in pairs( bar.Buttons[row] ) do
					
					local button      = bar.Buttons[row][col];
							
					if button.bindingType then

						local btnName     = button:GetName();
						local actionName  = button.bindingType..(button.actionID or "");
						local key1, key2  = GetBindingKey( actionName );
						
						if key1 then 
							SetBinding( key1, "CLICK "..btnName..":LeftButton" );
						end
						if key2 then 
							SetBinding( key2, "CLICK "..btnName..":LeftButton" );
						end
						
						button:SetScript( "OnEnter",
							function()
								
								nUI_ProfileStart( ProfileCounter, "OnEnter" );
	
								if nUI_Options.combat_tooltips or not InCombatLockdown() then
									
									local key1, key2  = GetBindingKey( "CLICK "..button:GetName()..":LeftButton" );
	
									ActionButton_SetTooltip( button );
									
									if key1 then GameTooltip:AddLine( nUI_L["Key Binding"].." 1: |cFF00FFFF"..GetBindingText( key1, "KEY_" ).."|r", 1, 1, 1 ); end
									if key2 then GameTooltip:AddLine( nUI_L["Key Binding"].." 2: |cFF00FFFF"..GetBindingText( key2, "KEY_" ).."|r", 1, 1, 1 ); end									
									if not key1 and not key2 then GameTooltip:AddLine( nUI_L["No key bindings found"], 1, 1, 1 ); end

									GameTooltip:AddLine( nUI_L["<ctrl-alt-right click> to change bindings"], 0, 1, 1 );
									GameTooltip:Show();
									
								end
								
								nUI_ProfileStop();
								
							end
						);
								
					end
				end
			end			
		end

		if KeyBindingFrame then
			
			if ( KeyBindingFrameCharacterButton:GetChecked() ) then
				KeyBindingFrame.which = CHARACTER_BINDINGS;
			else
				KeyBindingFrame.which = ACCOUNT_BINDINGS;
				if ( GetCurrentBindingSet() == CHARACTER_BINDINGS ) then
					if ( not CONFIRMED_DELETING_CHARACTER_SPECIFIC_BINDINGS ) then
						StaticPopup_Show("CONFIRM_DELETING_CHARACTER_SPECIFIC_BINDINGS");
						return;
					end
				end
			end
			
			SaveBindings(KeyBindingFrame.which);
		end
		
	-- intialize the button bars
	
	elseif event == "VARIABLES_LOADED" then
		
		nUI:patchConfig();
		nUI:setScale();
		
		-- set up the button bar options
		
		if not nUI_ButtonBarOptions then nUI_ButtonBarOptions = {}; end

		for name in pairs( nUI_DefaultConfig.ButtonBars ) do
				
			nUI_ButtonBars:configBar( name, use_default );
			
		end
		
		-- if we already have bars defined, then put them away before we build a new set
		
		for i in pairs( nUI_ButtonBars.Bars ) do
			
			local bar = nUI_ButtonBars.Bars[i];
			
			bar:UnregisterAllEvents();
			bar:Hide();
			bar:SetParent( nil );
			
			for j in pairs( bar.Buttons ) do
				bar.Buttons[j]:UnregisterAllEvents();
			end
		end
		
		-- create the button bars that are defined
		
		nUI_ButtonBars.Bars = {};
		
		for i in pairs( nUI_ButtonBarOptions ) do
			
			nUI_ButtonBars.Bars[i] = nUI_ButtonBars:createBar( i, nUI_ButtonBarOptions[i] );
					
		end

		-- set up the action bar state driver

		nUI_ActionBar = nUI_ButtonBars.Bars["nUI_ActionBar"];

		nUI_ActionBar:SetAttribute( "actionpage", 1 );
		
		nUI_ActionBar:Execute(
			[[
				ActionButtons = newtable( self:GetChildren() );
				actionType    = self:GetAttribute( "nUI_ActionType" );
			]]
		);

        nUI_ActionBar:SetAttribute( 
			"_onstate-page", 
			[[
			
				self:SetAttribute( "actionpage", tonumber( newstate ) );

				for i, button in ipairs( ActionButtons ) do
					
					button:SetAttribute( "touch", nil )
					
				end			
			]]
		);        
		
        nUI_ActionBar:SetAttribute( 
			"_onshow", 
			[[
				for i, button in ipairs( ActionButtons ) do
					
					button:SetAttribute( "touch", nil )
					
				end			
			]]
		);        
        
        if nUI_ActionBar.registered then 
			UnregisterStateDriver( nUI_ActionBar, "page" );
		end
		
--		RegisterStateDriver( nUI_ActionBar, "visibility", "[target=vehicle, exists] hide; show" );
		RegisterStateDriver( nUI_ActionBar, "page", "[bar:2] 2; [bar:3] 3; [bar:4] 4; [bar:5] 5; [bar:6] 6; [bonusbar:1] 7; [bonusbar:2] 8; [bonusbar:3] 9; [bonusbar:4] 10; [bonusbar:5] 11; 1" );

		nUI_ActionBar.registered = true;
		
	end
	
	nUI_ProfileStop();
	
end

nUI_ButtonBars:RegisterEvent( "VARIABLES_LOADED" );
nUI_ButtonBars:RegisterEvent( "PLAYER_LOGOUT" );
nUI_ButtonBars:RegisterEvent( "UPDATE_BINDINGS" );
nUI_ButtonBars:SetScript( "OnEvent", onButtonBarEvent );
	
-------------------------------------------------------------------------------
-- set configuration data for a single named bar

function nUI_ButtonBars:configBar( name, use_default )

	nUI_ProfileStart( ProfileCounter, "configBar" );
	
	local config  = nUI_ButtonBarOptions[name] or {};
	local default = nUI_DefaultConfig.ButtonBars[name] or config;

	if use_default then
		
		config.btn_size = default.btn_size;
		config.gap      = default.gap;
		config.anchor   = default.anchor;
		config.xOfs     = default.xOfs;
		config.yOfs     = default.yOfs;
		config.rows     = default.rows;
		config.cols     = default.cols;
		config.binding  = default.binding;
		config.label    = default.label;
		config.page     = default.page;
		config.base_id  = default.base_id;
				
	else
			
		config.btn_size = tonumber( config.btn_size or default.btn_size );
		config.gap      = tonumber( config.gap or default.gap );
		config.anchor   = strupper( config.anchor or default.anchor );
		config.xOfs     = tonumber( config.xOfs or default.xOfs );
		config.yOfs     = tonumber( config.yOfs or default.yOfs );
		config.rows     = tonumber( config.rows or default.rows );
		config.cols     = tonumber( config.cols or default.cols );
		config.binding  = config.binding or default.binding;
		config.nuibind  = config.nuibind or default.nuibind;
		config.label    = config.label or default.label;
		config.page     = config.page or default.page;
		config.base_id  = config.base_id or default.base_id;
		
	end
	
	if config.page then config.page = tonumber( config.page ); end
	if config.base_id then config.base_id = tonumber( config.base_id ); end
	
	nUI_ButtonBarOptions[name] = config;
	
	nUI_ProfileStop();
	
end

-------------------------------------------------------------------------------
-- create a new named button bar using the passed layout

function nUI_ButtonBars:createBar( name, layout )
	
	nUI_ProfileStart( ProfileCounter, "createBar" );
	
	local bar  = CreateFrame( "Frame", name, nUI_Dashboard.Anchor, "SecureHandlerStateTemplate" );
	bar.events = CreateFrame( "Frame", name.."_Events", nUI_ButtonBars );
	bar.layout = layout;
	
	bar:SetBackdrop(
		{
			bgFile   = "Interface\\AddOns\\nUI\\Art\\nUI_BevelboxBg.blp", 
			edgeFile = "Interface\\AddOns\\nUI\\Art\\nUI_BevelboxBorder.blp", 
			tile     = true, 
			tileSize = 1, 
			edgeSize = 12, 
			insets   = {left = 0, right = 0, top = 0, bottom = 0},
		}
	);
	
	bar:SetBackdropColor( 0, 0, 0, 0.35 );
	bar:SetAttribute( "nUI_ActionType", layout.binding );
	bar:SetAttribute( "nUI_BindingType", layout.nuibind );
	
	_G["BINDING_HEADER_"..layout.nuibind] = "nUI: "..layout.label;
	
	if layout.page then layout.base_id = (layout.page-1) * 12; end
	
	-- button initialization
	
	bar.Buttons = {};

	for i=1,layout.rows do
	
		bar.Buttons[i] = {};
		
		for j=1,layout.cols do
			
			local id         = (i-1)*layout.cols + j;
			local button     = CreateFrame( "CheckButton", "$parent_Button"..id, bar, "ActionBarButtonTemplate" );
			
			_G["BINDING_NAME_CLICK "..button:GetName()..":LeftButton"] = "Button #"..id;
			
			bar.Buttons[i][j] = button;
		
			button.action                   = layout.base_id + id;
			button.bindingType              = layout.binding;
			button.actionType               = layout.nuibind;
			button.actionID                 = id;
			button.nUI_CachedClearAllPoints = button.ClearAllPoints;
			button.nUI_CachedSetAllPoints   = button.SetAllPoints;
			button.nUI_CachedSetPoint       = button.SetPoint;

			button.bindAction = function()
				nUI_KeyBindingFrame.bindAction( button, bar.layout.label.." -- #"..id, "CLICK "..button:GetName()..":LeftButton" );
			end
		
			button:SetScript( "PreClick", 
				function( self, button, down )
					
					nUI_ProfileStart( ProfileCounter, "PreClick" );
	
					if not InCombatLockdown()
					and button == "RightButton" 
					and IsControlKeyDown() 
					and IsAltKeyDown()
					then
						self:SetAttribute( "alt-ctrl-type2", "bindAction" );
					end
					
					nUI_ProfileStop();
					
				end
			);
						
			button:SetScript( "PostClick", 
				function( self, button, down )
					
					nUI_ProfileStart( ProfileCounter, "PostClick" );
	
					if not InCombatLockdown()
					and button == "RightButton" 
					and IsControlKeyDown() 
					and IsAltKeyDown()
					then
						self:SetAttribute( "alt-ctrl-type2", nil );
						self:SetChecked( false );
					end
					
					nUI_ProfileStop();
					
				end
			);
						
			-- for the main action bar, we need the button ID set to support paging
			
			if name == "nUI_ActionBar" then
				bar:SetAttribute( "addchild", button );
				bar:SetFrameRef( "nUI_ActionBarButton"..id, button );
				button:SetID(id);
			else
				button:SetAttribute( "action", button.action ); 
			end			
		end
	end

	-- manage the layout of the bar
	
	bar.events:RegisterEvent( "DISPLAY_SIZE_CHANGED" );
	bar.events:RegisterEvent( "PLAYER_LOGIN" );
	bar.events:SetScript( "OnEvent", function() nUI_ButtonBars:layoutFrame( bar ); end );
		
	nUI_ProfileStop();
	
	return bar;
	
end

-------------------------------------------------------------------------------
-- actually perform the layout on the indicated bar. If no layout is passed
-- then the current layout for this bar is used. Otherwise the passed layout
-- will replace the current layout

function nUI_ButtonBars:layoutFrame( bar, layout )

	nUI_ProfileStart( ProfileCounter, "layoutFrame" );
	
	if layout then bar.layout = layout;
	else layout = bar.layout;
	end
	
	nUI:setScale();
				
	local btn_size    = layout.btn_size * nUI.scale;
	local btn_gap     = layout.gap * nUI.scale;
	
	bar:SetFrameStrata( nUI_Dashboard:GetFrameStrata() );
	bar:SetFrameLevel( nUI_Dashboard:GetFrameLevel()+2 );
	
	for row in pairs( bar.Buttons ) do
		
		for col in pairs( bar.Buttons[row] ) do
			
			local button      = bar.Buttons[row][col];

			-- save the binding we're using
	
			button:SetFrameStrata( bar:GetFrameStrata() );
			button:SetFrameLevel( bar:GetFrameLevel()+1 );
			button:SetScale( btn_size / button:GetHeight() );
	
			button:nUI_CachedClearAllPoints();
			
			if col == 1 and row == 1 then 
				button:nUI_CachedSetPoint( "TOPLEFT", bar, "TOPLEFT", btn_size * 0.05, -btn_size * 0.05 );
			elseif col == 1 then
				button:nUI_CachedSetPoint( "TOPLEFT", bar.Buttons[row-1][1], "BOTTOMLEFT", 0, -btn_gap );
			else 
				button:nUI_CachedSetPoint( "TOPLEFT", bar.Buttons[row][col-1], "TOPRIGHT", btn_gap, 0 );
			end
			
			nUI:initActionButton( button );
			
		end
	end		
		
	bar:SetHeight( layout.rows * btn_size + (layout.rows-1) * btn_gap + btn_size * 0.1 );
	bar:SetWidth( layout.cols * btn_size + (layout.cols-1) * btn_gap + btn_size * 0.1 );
	bar:SetPoint( layout.anchor, nUI_Dashboard, "CENTER", layout.xOfs * nUI.scale, layout.yOfs * nUI.scale );
	
	nUI_Movers:lockFrame( bar, true, layout.label );
	
	-- if this is the action bar, then add the paging buttons to it
	
	if bar:GetName() == "nUI_ActionBar" then

		ActionBarUpButton:SetParent( bar );
		ActionBarUpButton:SetFrameStrata( bar:GetFrameStrata() );
		ActionBarUpButton:SetFrameLevel( bar:GetFrameLevel() );
		ActionBarUpButton:SetScale( 1.5 * nUI.scale );
		ActionBarUpButton:ClearAllPoints();
		ActionBarUpButton:SetPoint( "TOPLEFT", bar, "TOPRIGHT", -10 * nUI.scale, 15 * nUI.scale );
		ActionBarUpButton:Show();
	
		ActionBarDownButton:SetParent( bar );
		ActionBarDownButton:SetFrameStrata( bar:GetFrameStrata() );
		ActionBarDownButton:SetFrameLevel( bar:GetFrameLevel() );
		ActionBarDownButton:SetScale( 1.5 * nUI.scale );
		ActionBarDownButton:ClearAllPoints();
		ActionBarDownButton:SetPoint( "BOTTOMLEFT", bar, "BOTTOMRIGHT", -10 * nUI.scale, -15 * nUI.scale  );
		ActionBarDownButton:Show();
		
	end
	
	nUI_ProfileStop();
	
end
