﻿--[[---------------------------------------------------------------------------

Copyright (c) 2008 by K. Scott Piel 
All Rights Reserved

E-mail: < kscottpiel@gmail.com >
Web:    < http://www.scottpiel.com >

This file is part of nUI.

    nUI is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    nUI is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with nUI.  If not, see <http://www.gnu.org/licenses/>.
	
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

--]]---------------------------------------------------------------------------

if not nUI_Profile then nUI_Profile = {}; end;

local CreateFrame = CreateFrame;

nUI_Profile.nUI_PossessBar = {};

local ProfileCounter = nUI_Profile.nUI_PossessBar;

-------------------------------------------------------------------------------
-- the possess bar frame

local anchor     = CreateFrame( "Frame", "nUI_PossessBarEvents", WorldFrame );
local frame      = CreateFrame( "Frame", "nUI_PossessBar", nUI_Dashboard.Anchor, "SecureFrameTemplate" );

-------------------------------------------------------------------------------
-- button initialization

frame.Buttons = {};

for i=1,10 do

	local button            = CreateFrame( "CheckButton", "$parent_Button"..i, frame, "PossessButtonTemplate" );	
	local btn_name          = button:GetName();
	frame.Buttons[i]        = button;	
	button.enabled          = true;
	button.layers           = {};
	button.layers.icon      = _G[btn_name.."Icon"];
	button.layers.cooldown  = _G[btn_name.."Cooldown"];
	
	button:SetID( i );
	button:SetChecked( 0 );
	button:SetNormalTexture( "" );
	
	button.SetNormalTexture = function() end;
	
	frame:SetAttribute( "addchild", button );
	
end

-------------------------------------------------------------------------------

frame.applyScale = function( scale )
	
	nUI_ProfileStart( ProfileCounter, "applyScale" );
	
	local scale   = scale or frame.scale or 1;
	local options = frame.options;
	
	if options and options.enabled then
		
		local btn_size = options.btn_size * scale * nUI.scale;
		local btn_gap  = (options.btn_gap or 0) * scale * nUI.scale;
		
		if frame.btn_size ~= btn_size 
		or frame.btn_gap  ~= btn_gap
		then
			
			frame.btn_size = btn_size;
			frame.btn_gap  = btn_gap;
			
			for i=1,#frame.Buttons do
				
				local button = frame.Buttons[i];
				
				button:ClearAllPoints();
				button:SetScale( btn_size / button:GetHeight() );
				
				if i == 1 then button:SetPoint( "TOPLEFT", frame, "TOPLEFT", 0, 0 );
				else button:SetPoint( "LEFT", frame.Buttons[i-1], "RIGHT", btn_gap, 0 );
				end
				
			end
						
			local width = btn_size * NUM_POSSESS_SLOTS + btn_gap * (NUM_POSSESS_SLOTS-1);
			
			if frame.width ~= width then
				frame.width = width;
				frame:SetHeight( btn_size );
				frame:SetWidth( width );
				frame:SetAttribute( "nUI_Width", width );
			end			
		end
	end
	
	nUI_ProfileStop();
	
end

-------------------------------------------------------------------------------

frame.applyOptions = function( options )

	nUI_ProfileStart( ProfileCounter, "applyOptions" );
	
	frame.options = options;
	
	if options and options.enabled then

		frame:SetParent( nUI_SpecialBars );
		frame:SetFrameStrata( nUI_SpecialBars:GetFrameStrata() );
		frame:SetFrameLevel( nUI_SpecialBars:GetFrameLevel()+1 );
		
		for i=1, #frame.Buttons do
			
			local button = frame.Buttons[i];
			button:SetFrameStrata( frame:GetFrameStrata() );
			button:SetFrameLevel( frame:GetFrameLevel()+1 );
		end

		frame.enabled = true;
	
		frame.applyScale( options.scale or frame.scale or 1 );
		
	end
	
	nUI_ProfileStop();
	
end

-------------------------------------------------------------------------------

frame.updateState = function( index )
	
	nUI_ProfileStart( ProfileCounter, "updateState" );
	
	local button = frame.Buttons[index];

	if index <= NUM_POSSESS_SLOTS then
		
		texture, name = GetPossessInfo( index );
	
		button:SetChecked( 0 );
		button.layers.icon:SetTexture( texture );	
		button.layers.icon:SetVertexColor( 1.0, 1.0, 1.0 );
	
		if not button.enabled then
			button.enabled = true;
			button:SetAlpha( 1 );
		end
		
	elseif button.enabled then
		
		button.enabled = false;
		button:SetAlpha( 0 );
	end	
	
	nUI_ProfileStop();
	
end

-------------------------------------------------------------------------------
-- manage the layout of the Possess Bar

anchor:SetScript( "OnEvent", 

	function() 

		nUI_ProfileStart( ProfileCounter, "OnEvent", event );
	
		if event == "PLAYER_ENTERING_WORLD"
		or event == "UPDATE_BONUS_ACTIONBAR"
		then
			
			frame.applyScale();
			
		end
		
		-- update the bar
		
		for i=1, #frame.Buttons do
			frame.updateState( i );				
		end
		
		nUI_ProfileStop();
		
	end 
);

anchor:RegisterEvent( "PLAYER_ENTERING_WORLD" );
anchor:RegisterEvent( "PLAYER_AURAS_CHANGED" );
anchor:RegisterEvent( "UPDATE_BONUS_ACTIONBAR" );
anchor:RegisterEvent( "SPELL_UPDATE_COOLDOWN" );
anchor:RegisterEvent( "SPELL_UPDATE_USABLE" );
